﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using Telerik.Windows.Controls;
using Telerik.Windows.Documents.FormatProviders;
using Telerik.Windows.Documents.FormatProviders.Xaml;
using Telerik.Windows.Documents.Model;
using Telerik.Windows.Documents.Model.Notes;
using Telerik.Windows.Documents.UI;

namespace MailMergeDemo
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        bool _IsPrinting = false;
        bool _IsPrintingStrated = false;
        int i = 0;
        private BackgroundWorker workerPrinting = null;
        private const string SampleDocumentPath = "SampleData/Template.xaml";
        List<TaxLines> _ListOfTaxLines = new List<TaxLines>();


        public MainWindow()
        {
            try
            {
                InitializeComponent();
                GetListOfTaxLines();

                this.Loaded += MainWindow_Loaded;
                this.radRichTextBox.Document.MailMergeDataSource.ItemsSource = _ListOfTaxLines;
                this.radRichTextBox.DocumentChanged += radRichTextBox_DocumentChanged;

                //workerPrinting = new BackgroundWorker();
                //workerPrinting.WorkerSupportsCancellation = true;
                //workerPrinting.WorkerReportsProgress = true;
                //workerPrinting.DoWork += workerPrinting_DoWork;
                //workerPrinting.ProgressChanged += workerPrinting_ProgressChanged;
                //workerPrinting.RunWorkerCompleted += workerPrinting_RunWorkerCompleted;

            }
            catch (Exception _Exception)
            {
                MessageBox.Show(_Exception.Message.ToString());
            }
        }

        void MainWindow_Loaded(object sender, RoutedEventArgs e)
        {
            try
            {
                this.radRichTextBox.Document = GetSampleDocument();
                //this.radRichTextBoxPrint.Document = GetSampleDocument();               
            }
            catch (Exception _Exception)
            {
                MessageBox.Show(_Exception.Message.ToString());
            }
        }

        void radRichTextBox_DocumentChanged(object sender, EventArgs e)
        {
            try
            {
                this.radRichTextBox.Document.MailMergeDataSource.ItemsSource = _ListOfTaxLines;
                this.radRichTextBox.UpdateAllFields(FieldDisplayMode.DisplayName);
            }
            catch (Exception _Exception)
            {
                MessageBox.Show(_Exception.Message.ToString());
            }
        }

        private void GetListOfTaxLines()
        {
            try
            {

                for (int i = 1; i <= 150; i++)
                {
                    TaxLines _TaxLines1 = new TaxLines();
                    _TaxLines1.LoanNo = "1234567_" + i;
                    _TaxLines1.BorrowerLastName = "JAJAL_" + i;
                    _TaxLines1.BorrowerAddress = "KANAK NAGAR, Near SANT KRUPA ASRAM " + i;
                    _TaxLines1.BorrowerCityStZip = "RAJKOT - 360005, GUJARAT" + i;
                    _TaxLines1.TaxAccountNo = "123AB-4670-111-258CC " + i;
                    _TaxLines1.Taxing_Authority = "RAJKOT TEXING " + i;
                    _TaxLines1.RecipAddress = "MOVDI CHOWKDI, OM PARK " + i;
                    _TaxLines1.RecipPhone = "111-222-3333 " + i;
                    _TaxLines1.AmtDue = 3738.77;
                    _TaxLines1.BankName = "TEST NEW COMPANY " + i;
                    _TaxLines1.AsOfDate = DateTime.Now;
                    _TaxLines1.BankAddress = "150 RING ROAD, NEAR BIG BAZZAR " + i;
                    _TaxLines1.BankCityStZip = "RAJKOT-360002 GUJRAT " + i;
                    _TaxLines1.BankPhone = "555-666-7777 " + i;
                    _ListOfTaxLines.Add(_TaxLines1);
                }
            }
            catch (Exception _Exception)
            {
                MessageBox.Show(_Exception.Message.ToString());
            }
        }

        private RadDocument GetSampleDocument()
        {
            IDocumentFormatProvider xamlProvider = new XamlFormatProvider();
            using (Stream stream = Application.GetResourceStream(GetResourceUri(SampleDocumentPath)).Stream)
            {
                return xamlProvider.Import(stream);
            }
        }

        private static Uri GetResourceUri(string resource)
        {
            AssemblyName assemblyName = new AssemblyName(typeof(MainWindow).Assembly.FullName);
            string resourcePath = "/" + assemblyName.Name + ";component/" + resource;
            Uri resourceUri = new Uri(resourcePath, UriKind.Relative);
            return resourceUri;
        }
        
        private Thread thread;
        private void btnPrintAll_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                XamlFormatProvider provider = new XamlFormatProvider();
                string editorDocument = provider.Export(this.radRichTextBox.Document);

                this.thread = new Thread(() =>
                     {
                         RadDocument document = provider.Import(editorDocument);
                         document.MailMergeDataSource.ItemsSource = _ListOfTaxLines;

                         // The MailMerge() method will merge all the records in the MailMergeDataSource. If you need to merge only a single record, you could use MailMergeCurrentRecord()
                         RadDocument result = document.MailMerge();
                         FreezeImages(result);
                         
                         this.printAction(result);
                     });

                thread.SetApartmentState(ApartmentState.STA);
                thread.Start();
            }
            catch (Exception _Exception)
            {
                MessageBox.Show(_Exception.Message.ToString());
            }
        }

        private Func<RadDocument, bool> printAction = new Func<RadDocument, bool>((doc) =>
        {

            RadRichTextBox richTextBox = new RadRichTextBox();
            richTextBox.Document = doc;


            PrintSettings settings = new PrintSettings();
            settings.UseDefaultPrinter = true;
            settings.DocumentName = "My Document";

            RadWindow window = new RadWindow()
            {
                Content = richTextBox,
                Opacity = 0,
                Height = 0,
                Width = 0
            };

            window.Show();
           
            richTextBox.Print(new PrintDialog(), settings);

            richTextBox.PrintCompleted += (s, e) =>
            {
                window.Close();
                richTextBox.Document = null;

            };

            return true;
        });
        #region Print Document

        //void workerPrinting_DoWork(object sender, DoWorkEventArgs e)
        //{
        //    try
        //    {
        //        //radRichTextBox.PrintCompleted += radRichTextBox_PrintCompleted;
        //        int _TotalRecords = _ListOfTaxLines.Count;
        //        while (_IsPrinting)
        //        {
        //            i++;
        //            int progressPercentage = Convert.ToInt32(((double)i / _TotalRecords) * 100);
        //            workerPrinting.ReportProgress(progressPercentage, i);
        //            System.Threading.Thread.Sleep(1000);
        //        }

        //    }
        //    catch (Exception _Exception)
        //    {
        //        MessageBox.Show(_Exception.Message.ToString());
        //    }
        //    finally
        //    {
        //        ((BackgroundWorker)sender).Dispose();
        //    }

        //}

        //void workerPrinting_ProgressChanged(object sender, ProgressChangedEventArgs e)
        //{
        //    try
        //    {
        //        if (_IsPrinting && _IsPrintingStrated == false)
        //        {
        //            _IsPrintingStrated = true;
        //            RadDocument _Document = this.radRichTextBoxPrint.Document;
        //            RadDocument _MergedDocument = this.radRichTextBoxPrint.MailMerge();
        //            if (_MergedDocument != null)
        //            {
        //                this.radRichTextBoxPrint.Document = _MergedDocument;
        //                PrintSettings _PrintSettings = new PrintSettings()
        //                {
        //                    DocumentName = "MailMergeDocument",
        //                    PrintMode = PrintMode.Native,
        //                    PrintScaling = PrintScaling.None,
        //                    //UseDefaultPrinter = true,
        //                    //PageRange = new PageRange(2, 4)
        //                };

        //                PrintDialog _PrintDialog = new PrintDialog();
        //                _PrintDialog.PageRangeSelection = PageRangeSelection.AllPages;
        //                _PrintDialog.UserPageRangeEnabled = true;
        //                Nullable<Boolean> _IsPrint = _PrintDialog.ShowDialog();
        //                if (_IsPrint == true)
        //                {
        //                    this.radRichTextBoxPrint.Print(_PrintDialog, _PrintSettings);
        //                }
        //                this.radRichTextBoxPrint.Document = _Document;
        //                _IsPrinting = false;
        //            }
        //        }

        //        //if (e.UserState != null)
        //        //    tbRecordsDone.Text = e.UserState + " / " + _ListOfTaxLines.Count;
        //        //pbCalculationProgress.Value = e.ProgressPercentage;
        //        //tbPercentageComplete.Text = e.ProgressPercentage + " % Completed";

        //    }
        //    catch (Exception _Exception)
        //    {
        //        MessageBox.Show(_Exception.Message.ToString());
        //    }
        //}

        //void workerPrinting_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        //{
        //    try
        //    {
        //        _IsPrintingStrated = false;

        //        //radRichTextBoxRibbonUI1.IsEnabled = true;
        //        //grdProgressBar.Visibility = Visibility.Collapsed;
        //        //pbCalculationProgress.Value = 0;
        //        //tbPercentageComplete.Text = "0 % Completed";
        //        //tbRecordsDone.Text = "";
        //    }
        //    catch (Exception _Exception)
        //    {
        //        MessageBox.Show(_Exception.Message.ToString());
        //    }
        //    finally
        //    {
        //        ((BackgroundWorker)sender).Dispose();
        //    }
        //}

        void radRichTextBox_PrintCompleted(object sender, PrintCompletedEventArgs e)
        {
            try
            {
                RadRichTextBox _RichTextBox = (sender as RadRichTextBox);
                if (_RichTextBox != null)
                {
                    TaxLines _Temptaxline = _RichTextBox.Document.MailMergeDataSource.CurrentItem as TaxLines;
                    MessageBox.Show("Loan No " + _Temptaxline.LoanNo + " printed successfully");
                }
            }
            catch (Exception _Exception)
            {
                MessageBox.Show(_Exception.Message.ToString());
            }
            finally
            {
                ((BackgroundWorker)sender).Dispose();
            }
        }
        #endregion

        private RadDocument FreezeImages(RadDocument document)
        {
            FreezeWatermarks(document);

            FreezeImageSources(document);

            foreach (RadDocument childDocument in GetAllChildDocuments(document))
            {
                FreezeImageSources(childDocument);
            }

            return document;
        }

        private void FreezeWatermarks(RadDocument doc)
        {
            foreach (Section section in doc.Sections)
            {
                FreezeWatermark(section.Headers.Default);

                FreezeWatermark(section.Headers.Even);

                FreezeWatermark(section.Headers.First);
            }
        }

        private void FreezeWatermark(Header header)
        {
            if (header != null && header.WatermarkSettings.ImageSettings != null && header.WatermarkSettings.ImageSettings.ImageSource != null)
            {
                header.WatermarkSettings.ImageSettings.ImageSource.Freeze();
            }
        }

        private void FreezeImageSources(RadDocument document)
        {
            foreach (var imageInline in document.EnumerateChildrenOfType<ImageInline>())
            {
                imageInline.ImageSource.Freeze();
            }

            foreach (var floatingImage in document.EnumerateChildrenOfType<FloatingImageBlock>())
            {
                floatingImage.ImageInline.ImageSource.Freeze();
            }
        }

        internal IEnumerable<RadDocument> GetAllChildDocuments(RadDocument document)
        {
            foreach (CommentRangeEnd commentRangeEnd in document.EnumerateChildrenOfType<CommentRangeEnd>())
            {
                if (commentRangeEnd.Comment != null && commentRangeEnd.Comment.Body != null)
                {
                    yield return commentRangeEnd.Comment.Body;
                }
            }

            foreach (NoteRangeEndBase noteRangeEnd in document.EnumerateChildrenOfType<NoteRangeEndBase>())
            {
                if (noteRangeEnd.Note != null && noteRangeEnd.Note.Body != null)
                {
                    yield return noteRangeEnd.Note.Body;
                }
            }

            foreach (var doc in GetAllHeaderFooterDocuments(document))
            {
                yield return doc;
            }
        }

        internal IEnumerable<RadDocument> GetAllHeaderFooterDocuments(RadDocument document)
        {
            foreach (Section section in document.Sections)
            {
                foreach (RadDocument headerFooterDocument in GetAllHeaderFooterDocuments(section))
                {
                    yield return headerFooterDocument;
                }
            }
        }

        private IEnumerable<RadDocument> GetAllHeaderFooterDocuments(Section section)
        {
            foreach (RadDocument document in GetAllNotNullHeadersFooters(section.Headers).Select(x => x.Body))
            {
                if (document != null)
                {
                    yield return document;
                }
            }

            foreach (RadDocument document in GetAllNotNullHeadersFooters(section.Footers).Select(x => x.Body))
            {
                if (document != null)
                {
                    yield return document;
                }
            }
        }

        internal IEnumerable<HeaderFooterBase> GetAllNotNullHeadersFooters<T>(HeadersFootersBase<T> headersFooters) where T : HeaderFooterBase, new()
        {
            if (headersFooters.First != null)
            {
                yield return headersFooters.First;
            }

            if (headersFooters.Even != null)
            {
                yield return headersFooters.Even;
            }

            if (headersFooters.Default != null)
            {
                yield return headersFooters.Default;
            }
        }

        private void btnStopPrintint_Click(object sender, RoutedEventArgs e)
        {
            this.thread.Abort();

            Thread.Sleep(1000);
            if (!thread.IsAlive)
            {
                MessageBox.Show("Printing stopped!");
            }
        }
    }
}